﻿Imports MigraDoc
Imports MigraDoc.DocumentObjectModel
Imports MigraDoc.Rendering
Imports System.IO

Public Class TestGenerator
    Public Property DocName As String
    Public Property SaveDirectoryPath As String
    Public Property Suffix As String
    Private Property Doc As Document

    Public Sub New()
    End Sub

    Public Async Function GenerateAsync() As Task
        Await Task.Run(Sub()
                           CreateDoc()
                           CreateFrontPage()
                           AddDayTable()
                           RenderAndSave()
                       End Sub)
    End Function

    Private Sub CreateDoc()
        Try
            Me.Doc = New Document
            Me.Doc.Info.Title = "Test-PDF"
            Me.Doc.Info.Author = "Migradoc-Forum-User: Silverfang"
        Catch ex As Exception
            Throw New Exception("Error-Occured at CreateDoc", ex)
        End Try
    End Sub

    Private Sub CreateFrontPage()
        Try
            Dim Sec As Section = Me.Doc.AddSection
            Dim Setup As PageSetup = Me.Doc.DefaultPageSetup.Clone

            With Setup
                .TopMargin = Unit.FromCentimeter(3.5)
                .LeftMargin = Unit.FromCentimeter(1)
                .RightMargin = Unit.FromCentimeter(1)
                .BottomMargin = Unit.FromCentimeter(1.5)
                .Orientation = Orientation.Landscape
                .PageFormat = PageFormat.A4
            End With

            Sec.PageSetup = Setup

            Dim Para As Paragraph = Sec.AddParagraph
            Para.Format.Alignment = ParagraphAlignment.Center
            Para.Format.SpaceBefore = Unit.FromCentimeter(3)
            Para.AddImage("Images\Firmenlogo.png")
            Para = Sec.AddParagraph("Bericht für den " & DateTime.Now.ToString(New System.Globalization.CultureInfo("de-DE")) & vbCrLf & "Für die Auswertungsgruppe ""Gruppe " & Me.Suffix & """")

            Para.Format.Font.Size = Unit.FromPoint(15)
            Para.Format.Font.Color = Colors.DarkGreen
            Para.Format.Font.Name = "Arial Black"
            Para.Format.Alignment = ParagraphAlignment.Center

            Para = Sec.Footers.Primary.AddParagraph("Generiert am: " & DateTime.Now.ToString("dd.MM.yyyy HH:mm:ss"))
            Para.Format.Alignment = ParagraphAlignment.Right
            Para.Format.Font.Size = Unit.FromPoint(8)
        Catch ex As Exception
            Throw New Exception("Error-Occured at CreateFrontPage", ex)
        End Try
    End Sub

    Private Sub AddDayTable()
        Try
            Dim Sec As Section = AddTableSection()
            Dim Para As Paragraph
            Para = Sec.Headers.FirstPage.AddParagraph("1. Testauswertung (" & DateTime.Now.ToString("ddd, dd.MM.yyyy") & ")")
            Para.Format.Alignment = ParagraphAlignment.Left
            Para.Format.Font.Color = Colors.DarkGreen
            Para.Format.Font.Size = Unit.FromPoint(12)
            Para.Format.Font.Name = "Arial Black"
            Dim Img As Shapes.Image

            Img = Sec.Headers.FirstPage.AddImage("Images\Firmenlogo.png")
            Img.Width = Unit.FromCentimeter(6)
            Img.LockAspectRatio = True
            Img.RelativeHorizontal = Shapes.RelativeHorizontal.Margin
            Img.RelativeVertical = Shapes.RelativeVertical.Page
            Img.Top = Shapes.ShapePosition.Top
            Img.Left = Shapes.ShapePosition.Right
            Img.WrapFormat.Style = Shapes.WrapStyle.Through

            Para.AddBookmark("Tagesumsatz")

            Para = Sec.Headers.Primary.AddParagraph("1. Testauswertung (" & DateTime.Now.ToString("ddd, dd.MM.yyyy") & ") Fortsetzung")
            Para.Format.Alignment = ParagraphAlignment.Left
            Para.Format.Font.Color = Colors.DarkGreen
            Para.Format.Font.Size = Unit.FromPoint(12)
            Para.Format.Font.Name = "Arial Black"

            Img = Sec.Headers.Primary.AddImage("Images\Firmenlogo.png")
            Img.Width = Unit.FromCentimeter(6)
            Img.LockAspectRatio = True
            Img.RelativeHorizontal = Shapes.RelativeHorizontal.Margin
            Img.RelativeVertical = Shapes.RelativeVertical.Page
            Img.Top = Shapes.ShapePosition.Top
            Img.Left = Shapes.ShapePosition.Right
            Img.WrapFormat.Style = Shapes.WrapStyle.Through

            Dim Tbl As Tables.Table = CreateBaseTable()
            Tbl = CreateTableHeader(Tbl, DateTime.Now.ToString("ddd dd.MM.yyyy"), DateTime.Now.AddDays(-364).ToString("ddd dd.MM.yyyy"))

            Dim Row As Tables.Row = Nothing
            For i As Integer = 1 To 10
                Row = Tbl.AddRow
                Row.Cells(0).AddParagraph(i)
                Row.Cells(0).Borders.Left.Width = 1
                Row.Cells(1).AddParagraph(i)
                Row.Cells(2).AddParagraph(i)
                Row.Cells(2).Borders.Left.Width = 1
                Row.Cells(3).AddParagraph((i - 1).ToString)

                Row.Cells(4).Borders.Right.Width = 1

                Row.Cells(5).AddParagraph(i)
                Row.Cells(6).AddParagraph(i)
                Row.Cells(7).AddParagraph(i)

                Row.Cells(7).Borders.Right.Width = 1

                Row.Cells(8).AddParagraph(i)
                Row.Cells(9).AddParagraph(i)
                Row.Cells(10).AddParagraph(0.5.ToString("#,##0.00 %"))

                Row.Cells(10).Borders.Right.Width = 1
            Next
            Tbl.Rows(Tbl.Rows.Count - 1).Borders.Bottom.Width = 1
            _Doc.LastSection.Add(Tbl)
        Catch ex As Exception
            Throw New Exception("Error-Occured at AddDayTable", ex)
        End Try
    End Sub

    Public Function CreateTableHeader(Tbl As Tables.Table, SubHeader1 As String, SubHeader2 As String) As Tables.Table
        Try
            Dim Row As Tables.Row
            Dim Cell As Tables.Cell

            Row = Tbl.AddRow
            Row.Format.Font.Bold = True
            Row.KeepWith = 2
            Row.HeadingFormat = True
            Row.Format.Alignment = ParagraphAlignment.Left

            Cell = Row.Cells(0)
            Cell.MergeRight = 10
            Cell.AddParagraph("Detailwerte")
            Cell.Borders.Width = 1
            Cell.Shading.Color = Colors.LightGreen

            Row = Tbl.AddRow
            Row.Format.Font.Bold = True
            Row.VerticalAlignment = Tables.VerticalAlignment.Center
            Row.Format.Alignment = ParagraphAlignment.Center

            Cell = Row.Cells(0)
            Cell.MergeDown = 1
            Cell.AddParagraph("Header1")
            Cell.Borders.Left.Width = 1
            Cell.Shading.Color = Colors.LightGreen

            Cell = Row.Cells(1)
            Cell.MergeDown = 1
            Cell.AddParagraph("Header2")
            Cell.Shading.Color = Colors.LightGreen

            Cell = Row.Cells(2)
            Cell.MergeRight = 1
            Cell.AddParagraph("Header3")
            Cell.Borders.Left.Width = 1
            Cell.Shading.Color = Colors.LightGreen

            Cell = Row.Cells(4)
            Cell.AddParagraph("Header4")
            Cell.MergeDown = 1
            Cell.Borders.Right.Width = 1
            Cell.Shading.Color = Colors.LightGreen

            Cell = Row.Cells(5)
            Cell.AddParagraph("Header5")
            Cell.MergeRight = 1
            Cell.Shading.Color = Colors.LightGreen

            Cell = Row.Cells(7)
            Cell.AddParagraph("Header6")
            Cell.MergeDown = 1
            Cell.Shading.Color = Colors.LightGreen
            Cell.Borders.Right.Width = 1

            Cell = Row.Cells(8)
            Cell.MergeRight = 1
            Cell.AddParagraph("Header7")
            Cell.Shading.Color = Colors.LightGreen

            Cell = Row.Cells(10)
            Cell.AddParagraph("Header8")
            Cell.MergeDown = 1
            Cell.Borders.Right.Width = 1
            Cell.Shading.Color = Colors.LightGreen

            Row = Tbl.AddRow
            Row.Format.Font.Bold = True
            Row.Format.Alignment = ParagraphAlignment.Center
            Row.Cells(2).AddParagraph(SubHeader1)
            Row.Cells(2).Shading.Color = Colors.LightGreen
            Row.Cells(3).AddParagraph(SubHeader2)
            Row.Cells(3).Shading.Color = Colors.LightGreen
            Row.Cells(5).AddParagraph(DateTime.Now.Year)
            Row.Cells(5).Shading.Color = Colors.LightGreen
            Row.Cells(6).AddParagraph(DateTime.Now.AddDays(-364).Year)
            Row.Cells(6).Shading.Color = Colors.LightGreen
            Row.Cells(8).AddParagraph(SubHeader1)
            Row.Cells(8).Shading.Color = Colors.LightGreen
            Row.Cells(9).AddParagraph(SubHeader2)
            Row.Cells(9).Shading.Color = Colors.LightGreen
            Row.Borders.Bottom.Width = 1

            Return Tbl
        Catch ex As Exception
            Throw New Exception("Error-occured at CreateDayTable", ex)
        End Try
    End Function

    Private Function AddTableSection() As Section
        Try
            Dim Sec As Section = _Doc.AddSection
            Dim Setup As PageSetup = _Doc.DefaultPageSetup.Clone

            Setup.Orientation = Orientation.Portrait
            Setup.PageFormat = PageFormat.A4
            Setup.DifferentFirstPageHeaderFooter = True

            With Setup
                .TopMargin = Unit.FromCentimeter(2.5)
                .LeftMargin = Unit.FromCentimeter(1.5)
                .RightMargin = Unit.FromCentimeter(1.5)
                .BottomMargin = Unit.FromCentimeter(1.5)
                .Orientation = Orientation.Landscape
                .PageFormat = PageFormat.A4
            End With

            Sec.PageSetup = Setup
            Sec.Footers.Primary.SetNull()

            Return Sec
        Catch ex As Exception
            Throw New Exception("Error-Occured at AddTableSection", ex)
        End Try
    End Function

    Private Sub AddNewPage()
        Try
            _Doc.LastSection.AddParagraph("")
            _Doc.LastSection.AddPageBreak()
        Catch ex As Exception
            Throw New Exception(".AddNewPage", ex)
        End Try
    End Sub

    Private Function CreateBaseTable() As Tables.Table
        Try
            Dim Tbl As New Tables.Table
            Dim Col As Tables.Column
            Tbl.Format.Font.Name = "Arial"
            Tbl.Rows.LeftIndent = 0
            Tbl.Borders.Width = 0.5
            Tbl.Format.Font.Size = Unit.FromPoint(8)

            Col = Tbl.AddColumn 'Filialnummer
            Col.Width = Unit.FromCentimeter(1.5)
            Col.Format.Alignment = ParagraphAlignment.Right

            Col = Tbl.AddColumn 'LadenID
            Col.Width = Unit.FromCentimeter(2.5)
            Col.Format.Alignment = ParagraphAlignment.Left

            Col = Tbl.AddColumn 'Umsatz Netto Akt
            Col.Width = Unit.FromCentimeter(2.9)
            Col.Format.Alignment = ParagraphAlignment.Right

            Col = Tbl.AddColumn 'Umsatz Netto Vorjahr
            Col.Width = Unit.FromCentimeter(2.9)
            Col.Format.Alignment = ParagraphAlignment.Right

            Col = Tbl.AddColumn 'Entw.
            Col.Width = Unit.FromCentimeter(2)
            Col.Format.Alignment = ParagraphAlignment.Right

            Col = Tbl.AddColumn 'Umsatz kumuliert Akt
            Col.Width = Unit.FromCentimeter(2.9)
            Col.Format.Alignment = ParagraphAlignment.Right

            Col = Tbl.AddColumn 'Umsatz kumuliert Vorjahr
            Col.Width = Unit.FromCentimeter(2.9)
            Col.Format.Alignment = ParagraphAlignment.Right

            Col = Tbl.AddColumn 'Entw.
            Col.Width = Unit.FromCentimeter(2)
            Col.Format.Alignment = ParagraphAlignment.Right

            Col = Tbl.AddColumn 'Kunden Akt
            Col.Width = Unit.FromCentimeter(2.9)
            Col.Format.Alignment = ParagraphAlignment.Right

            Col = Tbl.AddColumn 'Kunden Vorjahr
            Col.Width = Unit.FromCentimeter(2.9)
            Col.Format.Alignment = ParagraphAlignment.Right

            Col = Tbl.AddColumn 'Entw
            Col.Width = Unit.FromCentimeter(2)
            Col.Format.Alignment = ParagraphAlignment.Right

            Return Tbl
        Catch ex As Exception
            Throw New Exception(".CreateBaseTable", ex)
        End Try
    End Function

    Private Sub RenderAndSave()
        Try
            Dim Rend As New MigraDoc.Rendering.PdfDocumentRenderer
            Rend.Document = Me.Doc
            Rend.RenderDocument()
            If Not Directory.Exists(Me.SaveDirectoryPath) Then
                Directory.CreateDirectory(Me.SaveDirectoryPath)
            End If
            Rend.Save(Path.Combine(Me.SaveDirectoryPath, Me.DocName))
        Catch ex As Exception
            Throw New Exception(".RenderAndSave", ex)
        End Try
    End Sub
End Class
